@@ -1,40 +1,3 @@
-/*
- * T.C.F.S. 2.0 Alpha 1 
- *
- *      	   This  program  handles  RPC  "NFS"  data  requests
- *              adopting a secure transfer protocol.
- *                 This   is  an  unsecure   and  unchecked  version,
- *              use at your own risk.
- *
- *              Please, report Bugs to: <tcfs@edu-gw.dia.unisa.it>
- *
- * Authors:	Giuseppe Cattaneo, <cattaneo@udsab.dia.unisa.it>
- *		Giuseppe Persiano, <giuper@udsab.dia.unisa.it>
- *		Andrea Cozzolino, <andcoz@edu-gw.dia.unisa.it>
- *		Angelo Celentano, <angcel@edu-gw.dia.unisa.it>
- *		Aniello Del Sorbo, <anidel@edu-gw.dia.unisa.it>
- *		Ermelindo Mauriello, <ermmau@edu-gw.dia.unisa.it>
- *		Raffaele Pisapia, <rafpis@edu-gw.dia.unisa.it>
- *
- *   Permission to  use, copy, and modify  this software  without fee
- * is hereby granted, provided that this entire notice is included in
- * all copies  of  any  software  which  is  or  includes a  copy  or
- * modification of this software and in all copies  of the supporting
- * documentation for such software.
- *
- *   This  software is  distribuited  under  the  GNU General  Public
- * License  (version  2, June  1991). Check  the  file  'COPYING'  for
- * more  infos. Some  parts of  this  software  derive  from the  NFS
- * implementation in the Linux kernel 2.0.x.
- *
- * This software  maybe be used  for any  purpose provided  the above
- * copyright  notice  is retained.  It  is  supplied  as is,  with no
- * warranty expressed or implied.
- *
- */
-
-/* -+-_== */
-
 /*
  * Defining TCFS_PROC_DEBUG causes a lookup of a file named
  * "xyzzy" to toggle debugging.  Just cd to an TCFS-mounted
@@ -58,7 +21,7 @@
 #include <linux/pagemap.h>
 
 #include "xattr_client.h"
-#include "cypitfc.h"
+#include "desitfc.h"
 #include <asm/segment.h>
 
 #ifdef TCFS_PROC_DEBUG
@@ -84,10 +47,7 @@ static int *tcfs_tcrpc_verify(int *p);
 static int tcfs_stat_to_errno(int stat);
 
 /* These are the function needed to handle keys hash table */
-extern struct hash_entry ** hash_table;
-extern void init_hash(void);
-extern struct hash_entry * hash_add(int uid,char *deskey, void *other_data);
-extern struct hash_entry * hash_lookup(int uid);
+#include "hash.h"
 
 static int *xattr_tcrpc_header(int *p, int procedure, int ruid);
 int xattr_tcrpc_call(struct tcfs_server *server, int *start, int *end, int size);
@@ -321,7 +281,7 @@ retry:
 static struct tcfs_entry *cc_entry = NULL;
 extern int tcfs_lookup(struct inode *dir, const char *__name, int len,struct inode **result);
 
-int tcfs_proc_cryptdir(struct inode *inode, char mode,struct file *filp,struct hash_entry *htmp)
+int tcfs_proc_cryptdir(struct inode *inode, char mode,struct file *filp,des_key_schedule *ks)
 {
 	static kdev_t c_dev = 0;
 	static int c_ino;
@@ -414,12 +374,18 @@ int tcfs_proc_cryptdir(struct inode *inode, char mode,struct file *filp,struct h
 			strcpy(newname,tmp);
 			index=strlen(newname)+1;
 			if (mode!=0) { /* Removing secure flag */
-				kencrypt(newname,(index &0xfff8)+8,htmp->ks);
+				kendes(newname,(index &0xfff8)+8,ks);
 				tcfsencode(newname,(index &0xfff8)+8);
+#ifdef DEBUG_TCFS
+				printk ("TCFS: tcfsencode returned: %s\n", newname);
+#endif
 			}
 			else {
 				index=tcfsdecode(newname,index);
-				kdecrypt(newname,(index & 0xfff8)+8,htmp->ks);
+#ifdef DEBUG_TCFS
+				printk ("TCFS: tcfsdecode returned: %s\n", newname);
+#endif
+				kdedes(newname,(index & 0xfff8)+8,ks);
 			}
 			ret=tcfs_proc_rename(TCFS_SERVER(inode),TCFS_FH(inode),tmp,TCFS_FH(inode),newname,0);
 			if (ret<0) {
@@ -458,7 +424,7 @@ int tcfs_truncate(char *path,unsigned long lenght,struct tcfs_server *server)
 	return status;
 }
 
-int tcfs_endes(struct inode *inode,int mode,struct file *filp)
+int tcfs_endes(struct inode *inode,int mode,struct file *filp,char grp)
 {
 	struct tcfs_server *server=NULL;
 	struct tcfs_fh *fhandle;
@@ -467,19 +433,30 @@ int tcfs_endes(struct inode *inode,int mode,struct file *filp)
 	unsigned long off,size;
 	char *data=NULL;
 	struct hash_entry *htmp;
+	struct gid_hash_entry *ghtmp;
+	des_key_schedule *ks;
 
 #ifdef DEBUG_TCFS
-	printk("TCFS: tcfs_endes\n");
+	printk("TCFS: tcfs_endes, mode: %u\n", mode);
 #endif
-	htmp=hash_lookup(current->uid);
-	if (htmp==NULL) {
+	if (grp) {
+		ghtmp=gid_hash_lookup(inode->i_gid);
+		if (ghtmp==NULL || ghtmp->ks==NULL) {
+			return -EACCES;
+		}
+		ks=ghtmp->ks;
+	} else {
+		htmp=hash_lookup(current->uid);
+		if (htmp==NULL) {
 #ifdef DEBUG_TCFS
 	printk("TCFS: hash_lookup returned NULL\n");
 #endif
-		return -EACCES;
+			return -EACCES;
+		}
+		ks=htmp->ks;
 	}
 	if (S_ISDIR(inode->i_mode)) {
-		return tcfs_proc_cryptdir(inode,mode,filp,htmp);
+		return tcfs_proc_cryptdir(inode,mode,filp,ks);
 	} 
 	if (S_ISLNK(inode->i_mode)) {
 		return 0;
@@ -508,9 +485,9 @@ int tcfs_endes(struct inode *inode,int mode,struct file *filp)
 						if (read % 8!=0) {
 							read+=8-(read %8);
 						}
-						mkencrypt(data,read,htmp->ks);
+						mkendes(data,read,ks);
 					} else { /* Removing Secure Flag */
-						mkdecrypt(data,read,htmp->ks);
+						mkdedes(data,read,ks);
 						if (off+1024>=size) { 
 							/* We are at the end of file so we remove spure bytes */
 							if (inode->u.tcfs_i.tcfs_fl.bf.spure>0)
@@ -581,40 +558,110 @@ int tcfs_proc_seteattr(struct inode *inode,struct tcfs_server *server, char *fil
 {
 	int *p, *p0;
 	int status;
-	int ruid=0,error=0;
+	int ruid=0, error=0;
 	unsigned int oldflags;
+	unsigned int xf,xi,gf,gi;
+
+/*
+	if (inode == NULL) {
+#ifdef DEBUG_TCFS
+	printk("TCFS: tcfs_proc_seteattr-- inode == NULL\n");
+#endif 
+		return -EINVAL;
+	}
+*/
 
 #ifdef DEBUG_TCFS
 	printk("TCFS: tcfs_proc_seteattr\n");
 #endif 
-	if (inode!=NULL && 
-		(TCFS_IS_SECURE(cflag)!=TCFS_IS_SECURE(inode->u.tcfs_i.tcfs_fl.cflag))) /* CFLAG changed */
-		error=tcfs_endes(inode,TCFS_IS_SECURE(cflag),filp);
+
+
+	if (inode!=NULL) {
+		xf = TCFS_IS_SECURE(cflag);
+		xi = TCFS_IS_SECURE(inode->u.tcfs_i.tcfs_fl.cflag);
+		gf = cflag & TCFS_GSHARED_FL;
+		gi = TCFS_IS_SHARED(inode);
+
+		if (xf && gf){
+			#ifdef DEBUG_TCFS
+				printk("TCFS: tcfs_proc_seteattr-- Cannot set both G and X flag\n");
+			#endif 
+			return -EINVAL;
+		}
+
+		if ((xi==0) && (gi==0)){
+			if (xf){
+				error=tcfs_endes(inode,1,filp,0);
+			}
+
+			if (gf){
+				error=tcfs_endes(inode,1,filp,1);
+			}
+
+			if (!gf && !xf){
+				return 0;
+			}
+		}
+
+		if((xi==0) && gi){
+			if (gf==0){
+				error=tcfs_endes(inode,0,filp,1);
+			}
+			else return 0;
+		}
+
+		if(xi && (gi==0)){
+			if (xf==0){
+				error=tcfs_endes(inode,0,filp,0);
+			}
+			else return 0;
+		}
+		
+		if (xi && gi){
+			#ifdef DEBUG_TCFS
+			printk("TCFS: tcfs_proc_seteattr-- File has both G and X flag\n");
+			#endif 
+			return -EINVAL;
+		}
+	}
+
 	if (error <0)
 		return error;
+
 	if (!(p0 = tcfs_tcrpc_alloc(server->rsize)))
 		return -EIO;
+
 	if (inode!=NULL) {
+		/* L'effetto finale e' di aggiungere il numero di byte spuri
+		   nella variabile 'cflag' [nei bit a loro riservati] */
 		oldflags=inode->u.tcfs_i.tcfs_fl.cflag;
 		inode->u.tcfs_i.tcfs_fl.cflag=cflag;
-		inode->u.tcfs_i.tcfs_fl.bf.spure=error;
-		cflag=inode->u.tcfs_i.tcfs_fl.cflag;
-		inode->u.tcfs_i.tcfs_fl.cflag=oldflags;
+        	inode->u.tcfs_i.tcfs_fl.bf.spure=error;
+        	cflag=inode->u.tcfs_i.tcfs_fl.cflag;
+        	inode->u.tcfs_i.tcfs_fl.cflag=oldflags;
 	}
-	p= xattr_tcrpc_header(p0,2L,ruid);
-	p= xdr_encode_string(p,filename);
-	p= xdr_encode_eattr(p, &cflag);
+
+	p = xattr_tcrpc_header(p0,2L,ruid);
+	p = xdr_encode_string(p,filename);
+	p = xdr_encode_eattr(p, &cflag);
+
 	if ((status=tcfs_tcrpc_call(server, p0, p, server->rsize))<0) {
 		tcfs_tcrpc_free(p0);
 		return status;
 	} 
-	if (!(p = tcfs_tcrpc_verify(p0)))
-	status = -errno_TCFSERR_IO;
+
+	if (!(p = tcfs_tcrpc_verify(p0))){
+		status = -errno_TCFSERR_IO;
+	}
+
 	p = xdr_decode_eattr(p, &status); 
+	
 	if (inode!=NULL) {
 		inode->u.tcfs_i.tcfs_fl.cflag=cflag;
 	}
+
 	tcfs_tcrpc_free(p0);
+
 	return status;
 }    
 
@@ -626,6 +673,7 @@ int tcfs_proc_getattr(struct tcfs_server *server, struct tcfs_fh *fhandle,
 	int ruid = 0;
 
 	PRINTK("TCFS call  getattr\n");
+
 	if (!(p0 = tcfs_tcrpc_alloc(server->rsize)))
 		return -EIO;
 retry:
